<!DOCTYPE html>
<html>
  <head lang="en">
    <meta charset="UTF-8">
    <title>vcenter vm's</title>
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
  </head>
  <body>
    <div id="main"></div>
    <script src="{{ url_for('static', filename='js/bundle.js') }}"></script>
  </body>
</html>

